/**
 * Code Analyzer for Playwright MCP Server
 *
 * This module analyzes test files to extract reusable code patterns,
 * locators, and other elements to ensure generated tests follow
 * existing project structure and avoid code duplication.
 */
export interface TestFile {
    name: string;
    path: string;
    content: string;
    directory: string;
}
export interface CodePatterns {
    locators: Record<string, string>;
    fixtures: string[];
    helpers: Array<{
        name: string;
        code: string;
    }>;
    imports: string[];
    setupPatterns: string[];
    teardownPatterns: string[];
    assertionPatterns: string[];
    testStructure: {
        describe: string[];
        test: string[];
    };
    pageObjects: Array<{
        name: string;
        code: string;
    }>;
    commonImports: string[];
    fileStructure: Record<string, string[]>;
}
export interface TestLocation {
    directory: string;
    fileName: string;
    fullPath: string;
}
export declare class CodeAnalyzer {
    /**
     * Analyzes test files to extract reusable code patterns
     * @param workspaceRoot - Root directory of the workspace
     * @param pagePath - Path of the page being tested
     * @returns Object containing extracted code patterns
     */
    static analyzeTestFiles(workspaceRoot: string, pagePath: string): {
        testFiles: TestFile[];
        codePatterns: CodePatterns;
    };
    /**
     * Finds test files relevant to the specified page
     * @param workspaceRoot - Root directory of the workspace
     * @param pagePath - Path of the page being tested
     * @returns Array of relevant test files
     */
    static findRelevantTestFiles(workspaceRoot: string, pagePath: string): TestFile[];
    /**
     * Extracts code patterns from test files
     * @param testFiles - Array of test files to analyze
     * @returns Object containing extracted code patterns
     */
    private static extractCodePatterns;
    /**
     * Determines the best location for a new test file
     * @param codePatterns - Extracted code patterns
     * @param pagePath - Path of the page being tested
     * @returns Object containing the recommended file path and name
     */
    static determineTestLocation(codePatterns: CodePatterns, pagePath: string): TestLocation;
    /**
     * Analyzes locators to avoid duplication
     * @param existingLocators - Map of existing locators
     * @param html - HTML content of the page
     * @returns Map of recommended locators
     */
    static analyzeLocators(existingLocators: Record<string, string>, html: string): Record<string, string>;
    /**
     * Finds common patterns in file names
     * @param fileNames - Array of file names to analyze
     * @returns Common pattern or null if none found
     */
    private static findCommonPattern;
    /**
     * Generates a locator name from an element ID
     * @param id - Element ID
     * @returns Generated locator name
     */
    private static generateLocatorName;
}
