/**
 * Code Analyzer for Playwright MCP Server
 *
 * This module analyzes test files to extract reusable code patterns,
 * locators, and other elements to ensure generated tests follow
 * existing project structure and avoid code duplication.
 */
import fs from 'fs';
import path from 'path';
import { glob } from 'glob';
export class CodeAnalyzer {
    /**
     * Analyzes test files to extract reusable code patterns
     * @param workspaceRoot - Root directory of the workspace
     * @param pagePath - Path of the page being tested
     * @returns Object containing extracted code patterns
     */
    static analyzeTestFiles(workspaceRoot, pagePath) {
        console.log(`Analyzing test files for page: ${pagePath}`);
        // Find relevant test files
        const testFiles = this.findRelevantTestFiles(workspaceRoot, pagePath);
        console.log(`Found ${testFiles.length} relevant test files`);
        // Extract code patterns
        const codePatterns = this.extractCodePatterns(testFiles);
        return {
            testFiles,
            codePatterns
        };
    }
    /**
     * Finds test files relevant to the specified page
     * @param workspaceRoot - Root directory of the workspace
     * @param pagePath - Path of the page being tested
     * @returns Array of relevant test files
     */
    static findRelevantTestFiles(workspaceRoot, pagePath) {
        console.log(`Looking for test files relevant to: ${pagePath}`);
        // Extract page name from path for matching
        const pageKey = pagePath.split('/').filter(Boolean).pop() || 'home';
        console.log(`Page key for matching: ${pageKey}`);
        // Find test directories
        const testDirs = [
            path.join(workspaceRoot, 'tests'),
            path.join(workspaceRoot, 'test'),
            path.join(workspaceRoot, 'src', 'tests'),
            path.join(workspaceRoot, 'e2e')
        ];
        // Find files that might be relevant to this page
        const testFiles = [];
        for (const testDir of testDirs) {
            if (!fs.existsSync(testDir)) {
                console.log(`Test directory not found: ${testDir}`);
                continue;
            }
            console.log(`Searching for test files in: ${testDir}`);
            try {
                // Use glob to find all test files
                const testFilePaths = glob.sync(path.join(testDir, '**', '*.{spec,test}.{ts,js}'));
                for (const filePath of testFilePaths) {
                    const fileName = path.basename(filePath);
                    const fileContent = fs.readFileSync(filePath, 'utf8');
                    // Check if file is relevant to this page
                    if (fileName.includes(pageKey) || fileContent.includes(pageKey)) {
                        console.log(`Found relevant test file: ${fileName}`);
                        testFiles.push({
                            name: fileName,
                            path: filePath,
                            content: fileContent,
                            directory: path.dirname(filePath)
                        });
                    }
                }
            }
            catch (error) {
                console.error(`Error finding test files: ${error instanceof Error ? error.message : 'Unknown error'}`);
            }
        }
        // If we found too many files, limit to the most relevant ones
        if (testFiles.length > 5) {
            console.log(`Found ${testFiles.length} files, limiting to 5 most relevant`);
            return testFiles.slice(0, 5);
        }
        return testFiles;
    }
    /**
     * Extracts code patterns from test files
     * @param testFiles - Array of test files to analyze
     * @returns Object containing extracted code patterns
     */
    static extractCodePatterns(testFiles) {
        console.log('Extracting reusable code patterns from test files');
        const patterns = {
            locators: {},
            fixtures: [],
            helpers: [],
            imports: [],
            setupPatterns: [],
            teardownPatterns: [],
            assertionPatterns: [],
            testStructure: {
                describe: [],
                test: []
            },
            pageObjects: [],
            commonImports: [],
            fileStructure: {}
        };
        const imports = new Set();
        const commonImports = new Set();
        // Process each test file
        for (const file of testFiles) {
            // Extract locators
            const locatorRegex = /const\s+(\w+)\s*=\s*page\.locator\(['"]([^'"]+)['"]\)/g;
            let match;
            while ((match = locatorRegex.exec(file.content)) !== null) {
                patterns.locators[match[1]] = match[2];
            }
            // Extract imports
            const importRegex = /import\s+.*?from\s+['"]([^'"]+)['"]/g;
            while ((match = importRegex.exec(file.content)) !== null) {
                imports.add(match[0]);
                // Track common imports across files
                if (testFiles.length > 1) {
                    commonImports.add(match[0]);
                }
            }
            // Extract fixtures
            const fixtureRegex = /test\.use\(\{([^}]+)\}\)/g;
            while ((match = fixtureRegex.exec(file.content)) !== null) {
                patterns.fixtures.push(match[0]);
            }
            // Extract helper functions
            const helperFunctionRegex = /async\s+function\s+(\w+)\s*\([^)]*\)\s*\{[^}]*\}/g;
            while ((match = helperFunctionRegex.exec(file.content)) !== null) {
                patterns.helpers.push({
                    name: match[1],
                    code: match[0]
                });
            }
            // Extract setup patterns (beforeEach, beforeAll)
            const setupRegex = /(test\.beforeEach|test\.beforeAll)\(\s*async[^{]*\{[^}]*\}\s*\)/g;
            while ((match = setupRegex.exec(file.content)) !== null) {
                patterns.setupPatterns.push(match[0]);
            }
            // Extract teardown patterns (afterEach, afterAll)
            const teardownRegex = /(test\.afterEach|test\.afterAll)\(\s*async[^{]*\{[^}]*\}\s*\)/g;
            while ((match = teardownRegex.exec(file.content)) !== null) {
                patterns.teardownPatterns.push(match[0]);
            }
            // Extract assertion patterns
            const assertionRegex = /expect\([^)]+\)\.(toHaveText|toBeVisible|toHaveValue|toContainText|toHaveAttribute|toHaveClass|toHaveCount|toHaveCSS|toHaveId|toHaveURL|toHaveTitle|toBe|toEqual|toContain|not)\([^)]*\)/g;
            while ((match = assertionRegex.exec(file.content)) !== null) {
                patterns.assertionPatterns.push(match[0]);
            }
            // Extract test structure patterns
            const describeRegex = /test\.describe\(['"]([^'"]+)['"]/g;
            while ((match = describeRegex.exec(file.content)) !== null) {
                patterns.testStructure.describe.push(match[1]);
            }
            const testRegex = /test\(['"]([^'"]+)['"]/g;
            while ((match = testRegex.exec(file.content)) !== null) {
                patterns.testStructure.test.push(match[1]);
            }
            // Extract page objects
            const pageObjectRegex = /class\s+(\w+Page)\s*\{[^}]*\}/gs;
            while ((match = pageObjectRegex.exec(file.content)) !== null) {
                patterns.pageObjects.push({
                    name: match[1],
                    code: match[0]
                });
            }
            // Analyze file structure
            const directory = file.directory;
            if (!patterns.fileStructure[directory]) {
                patterns.fileStructure[directory] = [];
            }
            patterns.fileStructure[directory].push(file.name);
        }
        // Convert imports set to array
        patterns.imports = Array.from(imports);
        patterns.commonImports = Array.from(commonImports);
        // Deduplicate patterns
        patterns.fixtures = [...new Set(patterns.fixtures)];
        patterns.setupPatterns = [...new Set(patterns.setupPatterns)];
        patterns.teardownPatterns = [...new Set(patterns.teardownPatterns)];
        patterns.assertionPatterns = [...new Set(patterns.assertionPatterns)];
        patterns.testStructure.describe = [...new Set(patterns.testStructure.describe)];
        patterns.testStructure.test = [...new Set(patterns.testStructure.test)];
        console.log(`Extracted ${Object.keys(patterns.locators).length} locators and ${patterns.helpers.length} helper functions`);
        return patterns;
    }
    /**
     * Determines the best location for a new test file
     * @param codePatterns - Extracted code patterns
     * @param pagePath - Path of the page being tested
     * @returns Object containing the recommended file path and name
     */
    static determineTestLocation(codePatterns, pagePath) {
        // Extract page name from path
        const pageName = pagePath.split('/').filter(Boolean).pop() || 'home';
        // Find the most common directory structure
        const directories = Object.keys(codePatterns.fileStructure);
        const mostCommonDir = directories.length > 0 ? directories[0] : 'tests';
        // Generate file name
        const fileName = `${pageName}.spec.ts`;
        return {
            directory: mostCommonDir,
            fileName,
            fullPath: path.join(mostCommonDir, fileName)
        };
    }
    /**
     * Analyzes locators to avoid duplication
     * @param existingLocators - Map of existing locators
     * @param html - HTML content of the page
     * @returns Map of recommended locators
     */
    static analyzeLocators(existingLocators, html) {
        const recommendedLocators = {};
        // Extract IDs and classes from HTML
        const idRegex = /id=["']([^"']+)["']/g;
        const classRegex = /class=["']([^"']+)["']/g;
        let match;
        while ((match = idRegex.exec(html)) !== null) {
            const id = match[1];
            if (!Object.values(existingLocators).includes(id)) {
                recommendedLocators[this.generateLocatorName(id)] = `#${id}`;
            }
        }
        while ((match = classRegex.exec(html)) !== null) {
            const classes = match[1].split(' ');
            for (const className of classes) {
                if (!Object.values(existingLocators).includes(className)) {
                    recommendedLocators[this.generateLocatorName(className)] = `.${className}`;
                }
            }
        }
        return recommendedLocators;
    }
    /**
     * Finds common patterns in file names
     * @param fileNames - Array of file names to analyze
     * @returns Common pattern or null if none found
     */
    static findCommonPattern(fileNames) {
        if (fileNames.length === 0)
            return null;
        const patterns = new Set();
        for (const fileName of fileNames) {
            const parts = fileName.split('.');
            if (parts.length > 1) {
                patterns.add(parts[0]);
            }
        }
        return patterns.size === 1 ? Array.from(patterns)[0] : null;
    }
    /**
     * Generates a locator name from an element ID
     * @param id - Element ID
     * @returns Generated locator name
     */
    static generateLocatorName(id) {
        // Convert to camelCase
        return id
            .split(/[-_\s]/)
            .map((word, index) => index === 0
            ? word.toLowerCase()
            : word.charAt(0).toUpperCase() + word.slice(1).toLowerCase())
            .join('');
    }
}
//# sourceMappingURL=code-analyzer.js.map