/**
 * Test Structure Manager for Playwright MCP Server
 *
 * This module handles the integration of generated tests into the existing project structure,
 * ensuring they follow the same patterns and conventions as existing tests.
 */
interface TestLocation {
    directory: string;
    fileName: string;
    fullPath: string;
}
interface SaveTestResult {
    success: boolean;
    path?: string;
    message?: string;
    error?: string;
}
interface ProjectStructureAnalysis {
    testDirectories: string[];
    fixtureFiles: string[];
    helperFiles: string[];
    pageObjectFiles: string[];
    configFiles: string[];
    patterns: {
        fileNaming: NamingConvention | null;
        directoryOrganization: DirectoryOrganization | null;
        importPatterns: string[];
    };
}
type NamingConvention = 'kebab-case' | 'dot.case' | 'camelCase' | 'PascalCase';
type DirectoryOrganization = 'by-feature' | 'flat';
/**
 * Test Structure Manager class
 */
export declare class TestStructureManager {
    private workspaceRoot;
    constructor(workspaceRoot: string);
    /**
     * Saves a generated test to the appropriate location
     * @param testCode - The generated test code
     * @param testLocation - Object containing directory and fileName
     * @returns Result of the save operation
     */
    saveTest(testCode: string, testLocation: TestLocation): SaveTestResult;
    /**
     * Analyzes the project structure to determine test organization patterns
     * @returns Analysis of the project's test structure
     */
    analyzeProjectStructure(): ProjectStructureAnalysis;
    /**
     * Analyzes a test directory to extract patterns
     * @param directory - Directory to analyze
     * @param analysis - Analysis object to update
     */
    private analyzeTestDirectory;
    /**
     * Formats a test name according to the project's naming convention
     * @param testName - The raw test name
     * @param convention - The naming convention to use
     * @returns Formatted test name
     */
    private formatTestName;
    /**
     * Generates a test file path based on project structure analysis
     * @param pagePath - The path of the page being tested
     * @param analysis - Project structure analysis
     * @returns Recommended test location
     */
    generateTestPath(pagePath: string, analysis: ProjectStructureAnalysis): TestLocation;
}
export {};
