# Playwright MCP Server - Technical Documentation

This document provides technical details about how the Playwright MCP Server works and how to extend it.

## Architecture Overview

The Playwright MCP Server consists of several key components:

1. **Express Server**: Handles HTTP requests from Cursor
2. **Auth Manager**: Manages authentication configuration and login flows
3. **Code Analyzer**: Extracts reusable patterns from existing test files
4. **Test Structure Manager**: Analyzes project structure and determines test locations
5. **Page Identifier**: Determines which page the user is referring to
6. **Page Capturer**: Navigates to pages and captures their state
7. **MCP Generator**: Creates the Model Context Protocol payload

```
┌─────────────┐     ┌─────────────┐     ┌─────────────┐
│             │     │             │     │             │
│   Cursor    │────▶│  MCP Server │────▶│   OpenAI    │
│             │     │             │     │             │
└─────────────┘     └─────────────┘     └─────────────┘
                          │  ▲
                          │  │
                          ▼  │
┌─────────────┐     ┌─────────────┐     ┌─────────────┐
│             │     │             │     │             │
│ Site Context│◀───▶│  Playwright │────▶│  Test Files │
│             │     │             │     │             │
└─────────────┘     └─────────────┘     └─────────────┘
```

## Key Files and Their Functions

- **src/index.js**: Entry point that starts the server
- **src/server.js**: Main server logic and API endpoints
- **src/auth-manager.js**: Handles authentication configuration and login flows
- **src/code-analyzer.js**: Analyzes test files for reusable patterns
- **src/test-structure-manager.js**: Analyzes project structure and determines test locations
- **src/demo.js**: Helper script for creating sample files

## API Endpoints

### POST /mcp

This is the main endpoint that Cursor calls when you use the `/mcp playwright` command.

**Request Body:**
```json
{
  "prompt": "Create a test on the checkout page that validates the payment button",
  "workspaceRoot": "/path/to/your/project"
}
```

**Response:**
```json
{
  "mcp": "You are an AI test assistant skilled in Playwright automation..."
}
```

### POST /save-test

This endpoint saves a generated test to the appropriate location in the project.

**Request Body:**
```json
{
  "testCode": "import { test, expect } from '@playwright/test';...",
  "testLocation": {
    "directory": "tests",
    "fileName": "checkout.spec.ts",
    "fullPath": "tests/checkout.spec.ts"
  },
  "workspaceRoot": "/path/to/your/project"
}
```

**Response:**
```json
{
  "message": "Test saved to /path/to/your/project/tests/checkout.spec.ts",
  "path": "/path/to/your/project/tests/checkout.spec.ts"
}
```

### POST /auth-config

This endpoint gets or updates the authentication configuration.

**Request Body (Get):**
```json
{
  "workspaceRoot": "/path/to/your/project"
}
```

**Response (Get):**
```json
{
  "config": {
    "enabled": true,
    "loginUrl": "https://your-site.com/login",
    "usernameSelector": "#username",
    "passwordSelector": "#password",
    "submitSelector": "button[type='submit']",
    "username": "your-username",
    "password": "your-password",
    "requiresLogin": ["/account*", "/checkout", "*secure*"]
  }
}
```

**Request Body (Update):**
```json
{
  "workspaceRoot": "/path/to/your/project",
  "config": {
    "enabled": true,
    "loginUrl": "https://your-site.com/login",
    "usernameSelector": "#username",
    "passwordSelector": "#password",
    "submitSelector": "button[type='submit']",
    "username": "your-username",
    "password": "your-password",
    "requiresLogin": ["/account*", "/checkout", "*secure*"]
  }
}
```

**Response (Update):**
```json
{
  "message": "Authentication configuration updated successfully"
}
```

### POST /create-sample-auth

This endpoint creates a sample authentication configuration.

**Request Body:**
```json
{
  "workspaceRoot": "/path/to/your/project"
}
```

**Response:**
```json
{
  "message": "Sample authentication configuration created successfully",
  "config": {
    "enabled": false,
    "loginUrl": "https://example.com/login",
    "usernameSelector": "#username",
    "passwordSelector": "#password",
    "submitSelector": "button[type='submit']",
    "username": "",
    "password": "",
    "requiresLogin": ["/account*", "/checkout", "*secure*"]
  }
}
```

## Core Components

### Auth Manager

The Auth Manager handles authentication configuration and login flows.

```javascript
class AuthManager {
  constructor(workspaceRoot) {
    this.workspaceRoot = workspaceRoot;
    this.authConfigPath = path.join(workspaceRoot, '.playwright-mcp-auth.json');
    this.authConfig = this.loadAuthConfig();
  }

  loadAuthConfig() {
    // Load authentication configuration from file
  }

  saveAuthConfig(config) {
    // Save authentication configuration to file
  }

  requiresAuthentication(url) {
    // Check if a URL requires authentication
  }

  getAuthConfig() {
    // Get the current authentication configuration
  }

  createSampleAuthConfig() {
    // Create a sample authentication configuration
  }
}
```

### Code Analyzer

The Code Analyzer extracts reusable patterns from existing test files.

```javascript
function analyzeTestFiles(workspaceRoot, pagePath) {
  // Find relevant test files
  const testFiles = findRelevantTestFiles(workspaceRoot, pagePath);
  
  // Extract code patterns
  const codePatterns = extractCodePatterns(testFiles);
  
  return {
    testFiles,
    codePatterns
  };
}

function extractCodePatterns(testFiles) {
  // Extract locators, fixtures, helpers, imports, etc.
}

function analyzeLocators(existingLocators, html) {
  // Analyze locators to avoid duplication
}
```

### Test Structure Manager

The Test Structure Manager analyzes project structure and determines test locations.

```javascript
class TestStructureManager {
  constructor(workspaceRoot) {
    this.workspaceRoot = workspaceRoot;
  }

  saveTest(testCode, testLocation) {
    // Save a generated test to the appropriate location
  }

  analyzeProjectStructure() {
    // Analyze the project structure to determine test organization patterns
  }

  analyzeTestDirectory(directory, analysis) {
    // Analyze a test directory to extract patterns
  }

  formatTestName(testName, convention) {
    // Format a test name according to the project's naming convention
  }

  generateTestPath(pagePath, analysis) {
    // Generate a test file path based on project structure analysis
  }
}
```

### Page Capture

The page capture functionality navigates to pages and captures their state.

```javascript
async function capturePageState(url, authConfig = null) {
  const browser = await chromium.launch();
  const context = await browser.newContext();
  const page = await context.newPage();
  
  try {
    // Handle authentication if needed
    if (authConfig && authConfig.enabled) {
      await performLogin(page, authConfig);
    }
    
    // Navigate to the page
    await page.goto(url, { waitUntil: 'networkidle' });
    
    // Get HTML and screenshot
    const html = await page.content();
    const screenshotBuffer = await page.screenshot({ fullPage: true });
    const screenshot = screenshotBuffer.toString('base64');
    
    await browser.close();
    
    return { html, screenshot };
  } catch (error) {
    await browser.close();
    throw error;
  }
}

async function performLogin(page, authConfig) {
  // Navigate to login page
  await page.goto(authConfig.loginUrl, { waitUntil: 'networkidle' });
  
  // Fill login credentials
  await page.fill(authConfig.usernameSelector, authConfig.username);
  await page.fill(authConfig.passwordSelector, authConfig.password);
  
  // Submit login form
  await page.click(authConfig.submitSelector);
  
  // Wait for navigation to complete
  await page.waitForLoadState('networkidle');
}
```

### MCP Generation

The MCP generation functionality creates the Model Context Protocol payload.

```javascript
function generateMCP(
  pageInfo, 
  pageState, 
  testFiles, 
  prompt, 
  codePatterns, 
  requiresAuth,
  testLocation,
  locatorAnalysis,
  projectStructure
) {
  // Format elements, test files, code patterns, etc.
  
  // Build the MCP
  let mcp = `You are an AI test assistant skilled in Playwright automation...`;
  
  // Add authentication information if required
  if (requiresAuth) {
    mcp += `\n\nThis page requires authentication...`;
  }
  
  // Add user prompt and guidelines
  mcp += `\n\nUser Prompt: ${prompt}\n\nWrite a concise, complete Playwright test...`;
  
  return mcp;
}
```

## Extending the Server

### Adding AI-Powered Page Identification

For better page identification, you could enhance the `extractPageFromPrompt` function to use AI:

```javascript
async function extractPageFromPrompt(prompt, siteContext) {
  // Current implementation...
  
  // AI-powered identification
  const openai = getOpenAIClient();
  const completion = await openai.chat.completions.create({
    model: "gpt-3.5-turbo",
    messages: [
      {
        role: "system",
        content: `You are a page identification assistant. Given a list of pages and a user prompt, identify which page the user is referring to.`
      },
      {
        role: "user",
        content: `Pages: ${JSON.stringify(Object.keys(siteContext.pages))}\nPrompt: ${prompt}\nWhich page is the user referring to?`
      }
    ]
  });
  
  const aiSuggestion = completion.choices[0].message.content;
  // Parse AI suggestion and return the page...
}
```

### Adding Support for Test Execution

You could add a feature to execute the generated tests:

```javascript
async function executeTest(testCode, workspaceRoot) {
  // Create a temporary test file
  const testFilePath = path.join(workspaceRoot, 'temp-test.spec.ts');
  fs.writeFileSync(testFilePath, testCode);
  
  // Execute the test
  const { exec } = require('child_process');
  return new Promise((resolve, reject) => {
    exec(`cd ${workspaceRoot} && npx playwright test temp-test.spec.ts`, (error, stdout, stderr) => {
      if (error) {
        reject(error);
        return;
      }
      resolve({ stdout, stderr });
    });
  });
}
```

### Supporting Multiple Browsers

To support multiple browsers, modify the `capturePageState` function:

```javascript
async function capturePageState(url, authConfig = null, browserType = 'chromium') {
  // Get the browser instance based on type
  const browser = await getBrowser(browserType);
  const context = await browser.newContext();
  const page = await context.newPage();
  
  // Continue with page capture...
}

async function getBrowser(browserType) {
  const playwright = require('playwright');
  
  switch (browserType.toLowerCase()) {
    case 'firefox':
      return await playwright.firefox.launch();
    case 'webkit':
      return await playwright.webkit.launch();
    case 'chromium':
    default:
      return await playwright.chromium.launch();
  }
}
```

### Enhancing Code Pattern Extraction

To improve code pattern extraction, you could add more sophisticated analysis:

```javascript
function extractCodePatterns(testFiles) {
  // Current implementation...
  
  // Add more sophisticated pattern extraction
  const patterns = {
    // Current patterns...
    
    // Add pattern detection for common testing patterns
    testingPatterns: {
      dataProviders: extractDataProviders(testFiles),
      parameterizedTests: extractParameterizedTests(testFiles),
      conditionalTests: extractConditionalTests(testFiles)
    },
    
    // Add detection for custom test utilities
    customUtilities: extractCustomUtilities(testFiles)
  };
  
  return patterns;
}

function extractDataProviders(testFiles) {
  // Extract data provider patterns
}

function extractParameterizedTests(testFiles) {
  // Extract parameterized test patterns
}

function extractConditionalTests(testFiles) {
  // Extract conditional test patterns
}

function extractCustomUtilities(testFiles) {
  // Extract custom utility functions
}
```

## Performance Considerations

### Caching

To improve performance, you could add caching for page states:

```javascript
const pageStateCache = new Map();

async function getCachedPageState(url, authConfig = null, maxAge = 60000) {
  const cacheKey = `${url}-${authConfig ? 'auth' : 'noauth'}`;
  const cachedState = pageStateCache.get(cacheKey);
  
  if (cachedState && (Date.now() - cachedState.timestamp < maxAge)) {
    return cachedState.state;
  }
  
  const state = await capturePageState(url, authConfig);
  pageStateCache.set(cacheKey, {
    state,
    timestamp: Date.now()
  });
  
  return state;
}
```

### Rate Limiting

To avoid overwhelming the OpenAI API, add rate limiting:

```javascript
const requestQueue = [];
let isProcessing = false;

function queueRequest(fn) {
  return new Promise((resolve, reject) => {
    requestQueue.push({ fn, resolve, reject });
    processQueue();
  });
}

async function processQueue() {
  if (isProcessing || requestQueue.length === 0) {
    return;
  }
  
  isProcessing = true;
  const { fn, resolve, reject } = requestQueue.shift();
  
  try {
    const result = await fn();
    resolve(result);
  } catch (error) {
    reject(error);
  } finally {
    isProcessing = false;
    setTimeout(processQueue, 1000); // 1 second delay between requests
  }
}
```

## Security Considerations

### Input Validation

Always validate input from Cursor:

```javascript
function validateInput(req) {
  const { prompt, workspaceRoot } = req.body;
  
  if (!prompt || typeof prompt !== 'string') {
    throw new Error('Invalid prompt');
  }
  
  if (!workspaceRoot || typeof workspaceRoot !== 'string') {
    throw new Error('Invalid workspace root');
  }
  
  // Sanitize inputs
  const sanitizedPrompt = prompt.replace(/[<>]/g, '');
  const sanitizedWorkspaceRoot = workspaceRoot.replace(/[;|&]/g, '');
  
  return { prompt: sanitizedPrompt, workspaceRoot: sanitizedWorkspaceRoot };
}
```

### API Key Protection

Never log or expose the API key:

```javascript
function logConfig() {
  console.log('Configuration:');
  console.log(`- Port: ${process.env.PORT || 3000}`);
  console.log(`- API Key: ${process.env.OPENAI_API_KEY ? '[REDACTED]' : 'Not set'}`);
}
```

### Authentication Data Protection

Protect authentication data:

```javascript
function saveAuthConfig(config) {
  // Encrypt sensitive data
  const encryptedConfig = {
    ...config,
    username: encrypt(config.username),
    password: encrypt(config.password)
  };
  
  // Save encrypted config
  fs.writeFileSync(this.authConfigPath, JSON.stringify(encryptedConfig, null, 2));
}

function getAuthConfig() {
  // Decrypt sensitive data
  const config = this.loadAuthConfig();
  
  return {
    ...config,
    username: decrypt(config.username),
    password: decrypt(config.password)
  };
}

function encrypt(text) {
  // Implement encryption
}

function decrypt(text) {
  // Implement decryption
}
```

## Debugging

To enable debugging, add verbose logging:

```javascript
const DEBUG = process.env.DEBUG === 'true';

function log(message, data) {
  if (!DEBUG) return;
  
  console.log(`[${new Date().toISOString()}] ${message}`);
  if (data) {
    console.log(JSON.stringify(data, null, 2));
  }
}
```

## Testing

Here's how you could add tests for the server:

```javascript
// test/server.test.js
const { app } = require('../src/server');
const request = require('supertest');
const fs = require('fs');
const path = require('path');

// Mock dependencies
jest.mock('playwright', () => ({
  chromium: {
    launch: jest.fn().mockImplementation(() => ({
      newPage: jest.fn().mockImplementation(() => ({
        goto: jest.fn(),
        content: jest.fn().mockResolvedValue('<html><body>Test</body></html>'),
        screenshot: jest.fn().mockResolvedValue(Buffer.from('test')),
        close: jest.fn()
      })),
      close: jest.fn()
    }))
  }
}));

jest.mock('openai', () => ({
  OpenAI: jest.fn().mockImplementation(() => ({
    chat: {
      completions: {
        create: jest.fn().mockResolvedValue({
          choices: [{ message: { content: 'test' } }]
        })
      }
    }
  }))
}));

describe('MCP Server', () => {
  beforeEach(() => {
    // Create test site context
    const testSiteContext = {
      baseURL: 'https://example.com',
      pages: {
        '/': {
          title: 'Test',
          description: 'Test page',
          elements: { testButton: '#test' }
        }
      }
    };
    
    fs.writeFileSync(
      path.join(process.cwd(), 'test-site-context.json'),
      JSON.stringify(testSiteContext)
    );
  });
  
  afterEach(() => {
    // Clean up
    if (fs.existsSync(path.join(process.cwd(), 'test-site-context.json'))) {
      fs.unlinkSync(path.join(process.cwd(), 'test-site-context.json'));
    }
  });
  
  test('POST /mcp returns MCP payload', async () => {
    const response = await request(app)
      .post('/mcp')
      .send({
        prompt: 'Create a test for the homepage',
        workspaceRoot: process.cwd()
      });
    
    expect(response.status).toBe(200);
    expect(response.body).toHaveProperty('mcp');
    expect(typeof response.body.mcp).toBe('string');
  });
});
```

## Publishing to npm

To publish the package to npm:

1. Update the package.json file:

```json
{
  "name": "playwright-mcp-server",
  "version": "1.0.0",
  "description": "MCP server for Playwright test generation",
  "main": "src/index.js",
  "bin": {
    "playwright-mcp-server": "./src/index.js"
  },
  "scripts": {
    "start": "node src/index.js",
    "test": "jest"
  },
  "keywords": [
    "playwright",
    "testing",
    "mcp",
    "cursor",
    "automation"
  ],
  "author": "Your Name",
  "license": "MIT",
  "dependencies": {
    "express": "^4.18.2",
    "glob": "^8.1.0",
    "openai": "^4.0.0",
    "playwright": "^1.40.0"
  },
  "devDependencies": {
    "jest": "^29.0.0",
    "supertest": "^6.0.0"
  }
}
```

2. Create a .npmignore file:

```
test/
.github/
.vscode/
node_modules/
*.log
```

3. Login to npm:

```bash
npm login
```

4. Publish the package:

```bash
npm publish
```

## Conclusion

This technical documentation provides an overview of the Playwright MCP Server architecture, API endpoints, core components, and extension points. Use this as a reference when customizing or extending the server to meet your specific needs.
