# Playwright MCP Server - User Guide

This guide explains how to use the Playwright MCP Server to generate Playwright tests directly from Cursor's chat interface.

## What is the Playwright MCP Server?

The Playwright MCP Server is a custom Model Context Protocol (MCP) server that integrates with Cursor to provide AI-powered Playwright test generation. It allows you to:

1. Type natural language prompts in Cursor's chat
2. Automatically identify which page you're referring to
3. Capture the current state of that page (HTML and screenshot)
4. Analyze your existing test files to extract reusable code patterns
5. Determine the best location for the new test based on your project structure
6. Generate a complete Playwright test that follows your existing patterns

## Getting Started

### Prerequisites

- Node.js 14 or higher
- npm or yarn
- Cursor editor
- OpenAI API key
- Playwright installed in your project

### Installation

1. Extract the `playwright-mcp-server.zip` file
2. Navigate to the extracted directory:
   ```bash
   cd playwright-mcp-server
   ```
3. Install dependencies:
   ```bash
   npm install
   ```
4. Make the server executable:
   ```bash
   chmod +x src/index.js
   ```
5. (Optional) Link the package globally for easier access:
   ```bash
   npm link
   ```

### Setting Up Your OpenAI API Key

The MCP server requires an OpenAI API key to generate tests. Set it as an environment variable:

```bash
export OPENAI_API_KEY="your-api-key"
```

For Windows:
```cmd
set OPENAI_API_KEY=your-api-key
```

To make this permanent, add it to your shell profile (~/.bashrc, ~/.zshrc, etc.).

### Creating Your Site Context File

The server needs a site context file to understand your website structure. This file should be named `site-context.json` and placed in your project root.

If you don't have one yet, you can create a sample:
```bash
node src/demo.js create-sample
```

Then customize it with your actual website structure:
```json
{
  "baseURL": "https://your-site.com",
  "pages": {
    "/": {
      "title": "Homepage",
      "description": "Main landing page",
      "elements": {
        "loginButton": "#login-btn",
        "signupLink": ".signup-link"
      }
    },
    "/checkout": {
      "title": "Checkout",
      "description": "Payment processing page",
      "elements": {
        "paymentButton": "#pay-now-btn",
        "cardNumberField": "input[name='card-number']"
      }
    }
  }
}
```

### Setting Up Authentication

For pages that require authentication, create a `.playwright-mcp-auth.json` file in your project root:

```json
{
  "enabled": true,
  "loginUrl": "https://your-site.com/login",
  "usernameSelector": "#username",
  "passwordSelector": "#password",
  "submitSelector": "button[type='submit']",
  "username": "your-username",
  "password": "your-password",
  "requiresLogin": [
    "/account*",
    "/checkout",
    "*secure*"
  ]
}
```

The `requiresLogin` array supports patterns:
- `/account*` - Matches any path starting with "/account"
- `*secure*` - Matches any path containing "secure"
- `/checkout` - Matches the exact path "/checkout"

### Configuring Cursor

Create or update `.cursor/settings.json` in your project:

```json
{
  "mcpServers": {
    "playwright": {
      "command": "npx",
      "args": [
        "playwright-mcp-server"
      ]
    }
  }
}
```

If you didn't use `npm link`, specify the path to the server:

```json
{
  "mcpServers": {
    "playwright": {
      "command": "node",
      "args": [
        "/path/to/playwright-mcp-server/src/index.js"
      ]
    }
  }
}
```

## Using the MCP Server

### In Cursor Chat

Once configured, you can use it in Cursor's chat:

```
/mcp playwright Create a test on the checkout page that validates the payment button
```

The server will:
1. Identify that you're referring to the checkout page
2. Navigate to that page to capture its current state
3. Analyze your existing test files for reusable patterns
4. Determine the best location for the new test
5. Generate a complete Playwright test that follows your existing patterns

### Example Prompts

Here are some example prompts you can use:

- "Create a test on the homepage that checks if the login button is visible"
- "Generate a test for the checkout page that fills in payment details and submits the form"
- "Write a test that navigates from the product page to the cart and verifies the item was added"
- "Create a test that logs in, adds an item to cart, and completes checkout"

### Tips for Effective Prompts

1. **Be specific about the page**: Mention the page name explicitly
2. **Describe the actions clearly**: List the steps in order
3. **Specify what to verify**: Mention what assertions to include
4. **Reference existing elements**: Use names from your site context

## Code Reuse Features

The MCP server analyzes your existing test files to extract reusable patterns:

### Locators

The server extracts locators from your existing tests:

```typescript
const loginButton = page.locator('#login-btn');
```

And reuses them in generated tests to avoid duplication.

### Helper Functions

The server identifies helper functions in your tests:

```typescript
async function login(page, username, password) {
  await page.fill('#username', username);
  await page.fill('#password', password);
  await page.click('button[type="submit"]');
}
```

And suggests using them in generated tests.

### Page Objects

The server recognizes page object classes:

```typescript
class LoginPage {
  constructor(page) {
    this.page = page;
    this.usernameInput = page.locator('#username');
    this.passwordInput = page.locator('#password');
    this.submitButton = page.locator('button[type="submit"]');
  }
  
  async login(username, password) {
    await this.usernameInput.fill(username);
    await this.passwordInput.fill(password);
    await this.submitButton.click();
  }
}
```

And incorporates them into generated tests when appropriate.

### Test Structure

The server analyzes your test structure:

```typescript
test.describe('Login functionality', () => {
  test.beforeEach(async ({ page }) => {
    await page.goto('https://example.com');
  });
  
  test('should login successfully', async ({ page }) => {
    // Test implementation
  });
});
```

And ensures generated tests follow the same patterns.

## Project Structure Integration

The MCP server analyzes your project structure to determine:

### File Naming Conventions

- kebab-case: `login-page.spec.ts`
- camelCase: `loginPage.spec.ts`
- PascalCase: `LoginPageSpec.ts`
- dot.case: `login.page.spec.ts`

### Directory Organization

- Flat structure: All tests in a single directory
- By-feature: Tests organized in subdirectories by feature
- By-page: Tests organized by page or component

The server ensures generated tests follow your project's conventions.

## Troubleshooting

### Server Won't Start

- Make sure you have Node.js installed
- Check that all dependencies are installed with `npm install`
- Verify that the server file is executable with `chmod +x src/index.js`

### API Key Issues

- Ensure your OpenAI API key is set: `export OPENAI_API_KEY="your-api-key"`
- Check that the key is valid and has sufficient credits

### Cursor Integration Issues

- Verify your `.cursor/settings.json` file is correctly formatted
- Make sure Cursor can find the MCP server executable
- Try restarting Cursor after making changes

### Test Generation Issues

- Check that your site context file is valid JSON
- Ensure your site is accessible from the server
- Make your prompts more specific if the wrong page is being identified
- Check the server logs for error messages

## Advanced Usage

### Testing with Authentication

For pages that require authentication:

1. Create a `.playwright-mcp-auth.json` file as described above
2. Ensure the `enabled` field is set to `true`
3. Add the paths that require authentication to the `requiresLogin` array

The server will automatically handle the login process before capturing the page state.

### Handling Dynamic Content

For pages with dynamic content:

1. Be specific about the elements you want to interact with
2. Reference stable selectors from your site context
3. Include waiting or assertion logic in your prompts

### Customizing the MCP Server

You can customize the server by modifying the source code:

- `src/server.js`: Main server logic
- `src/auth-manager.js`: Authentication handling
- `src/code-analyzer.js`: Code pattern extraction
- `src/test-structure-manager.js`: Project structure analysis

See the TECHNICAL.md file for more details on extending the server.

## Getting Help

If you encounter any issues:

1. Check the console output for error messages
2. Refer to the README.md file for troubleshooting tips
3. Check the Playwright documentation for test-specific issues

## Next Steps

- Create a comprehensive site context file for your website
- Set up authentication if your site requires login
- Experiment with different types of test prompts
- Integrate the generated tests into your CI/CD pipeline
