/**
 * Authentication Manager for Playwright MCP Server
 * 
 * This module handles authentication for pages that require login.
 * It supports storing and managing authentication configurations,
 * and provides functions for logging in during page capture.
 */

import fs from 'fs';
import path from 'path';
import { AuthConfig } from './types.js';

// Default location for auth config file
const DEFAULT_AUTH_CONFIG_PATH = '.playwright-mcp-auth.json';

/**
 * Authentication Manager class
 */
export class AuthManager {
  private workspaceRoot: string;
  private authConfigPath: string;
  private authConfig: AuthConfig;

  constructor(workspaceRoot: string) {
    this.workspaceRoot = workspaceRoot;
    this.authConfigPath = path.join(workspaceRoot, DEFAULT_AUTH_CONFIG_PATH);
    this.authConfig = this.loadAuthConfig();
  }

  /**
   * Load authentication configuration from file
   */
  private loadAuthConfig(): AuthConfig {
    try {
      if (fs.existsSync(this.authConfigPath)) {
        const configData = fs.readFileSync(this.authConfigPath, 'utf8');
        return JSON.parse(configData);
      }
    } catch (error) {
      console.error(`Error loading auth config: ${error instanceof Error ? error.message : 'Unknown error'}`);
    }
    
    // Return empty config if file doesn't exist or has errors
    return {
      enabled: false,
      loginUrl: '',
      usernameSelector: '',
      passwordSelector: '',
      submitSelector: '',
      username: '',
      password: '',
      requiresLogin: []
    };
  }

  /**
   * Save authentication configuration to file
   */
  public saveAuthConfig(config: AuthConfig): boolean {
    try {
      fs.writeFileSync(this.authConfigPath, JSON.stringify(config, null, 2));
      this.authConfig = config;
      return true;
    } catch (error) {
      console.error(`Error saving auth config: ${error instanceof Error ? error.message : 'Unknown error'}`);
      return false;
    }
  }

  /**
   * Check if a URL requires authentication
   */
  public requiresAuthentication(url: string): boolean {
    if (!this.authConfig.enabled) {
      return false;
    }

    // Check if the URL matches any of the patterns in requiresLogin
    return this.authConfig.requiresLogin.some((pattern: string) => {
      if (pattern.startsWith('*')) {
        return url.includes(pattern.substring(1));
      } else if (pattern.endsWith('*')) {
        return url.startsWith(pattern.substring(0, pattern.length - 1));
      } else {
        return url === pattern || url.includes(pattern);
      }
    });
  }

  /**
   * Get authentication configuration
   */
  public getAuthConfig(): AuthConfig {
    return this.authConfig;
  }

  /**
   * Create a sample authentication configuration
   */
  public createSampleAuthConfig(): boolean {
    const sampleConfig: AuthConfig = {
      enabled: false,
      loginUrl: 'https://example.com/login',
      usernameSelector: '#username',
      passwordSelector: '#password',
      submitSelector: 'button[type="submit"]',
      username: '',
      password: '',
      requiresLogin: [
        '/account*',
        '/checkout',
        '*secure*'
      ]
    };

    return this.saveAuthConfig(sampleConfig);
  }
} 