#!/usr/bin/env node

/**
 * This script demonstrates how to use the Playwright MCP Server
 * with a sample site context and test generation.
 */

import fs from 'fs';
import path from 'path';
import { AuthManager } from './auth-manager.js';
import { SiteContext } from './types.js';

// Sample site context
const sampleSiteContext: SiteContext = {
  baseURL: "https://example.com",
  pages: {
    "/": {
      title: "Example Domain",
      description: "Example Domain homepage",
      elements: {
        heading: "h1",
        paragraph: "p",
        moreInfoLink: "a"
      }
    },
    "/about": {
      title: "About Example",
      description: "About page for Example Domain",
      elements: {
        aboutHeading: "h1",
        aboutContent: ".about-content",
        contactButton: "#contact-btn"
      }
    }
  }
};

// Create sample site context file
function createSampleSiteContext(): void {
  const filePath = path.join(process.cwd(), 'sample-site-context.json');
  fs.writeFileSync(filePath, JSON.stringify(sampleSiteContext, null, 2));
  console.log(`Created sample site context at: ${filePath}`);
}

// Create sample authentication configuration
function createSampleAuthConfig(): void {
  const workspaceRoot = process.cwd();
  const authManager = new AuthManager(workspaceRoot);
  const success = authManager.createSampleAuthConfig();
  
  if (success) {
    console.log(`Created sample authentication configuration at: ${workspaceRoot}/.playwright-mcp-auth.json`);
  } else {
    console.error('Failed to create sample authentication configuration');
  }
}

// Print usage instructions
function printUsage(): void {
  console.log(`
Playwright MCP Server - Demo Script

This script helps you get started with the Playwright MCP Server.

Usage:
  node demo.js [command]

Commands:
  create-sample         Create a sample site context file
  create-sample-auth    Create a sample authentication configuration
  help                  Show this help message

Examples:
  node demo.js create-sample         Creates a sample site context file
  node demo.js create-sample-auth    Creates a sample authentication configuration
  node demo.js help                  Shows this help message
  `);
}

// Main function
function main(): void {
  const command = process.argv[2];

  if (!command || command === 'help') {
    printUsage();
    return;
  }

  if (command === 'create-sample') {
    createSampleSiteContext();
    return;
  }
  
  if (command === 'create-sample-auth') {
    createSampleAuthConfig();
    return;
  }

  console.log(`Unknown command: ${command}`);
  printUsage();
}

// Run the script
main(); 