#!/usr/bin/env node

/**
 * This is the entry point for the Playwright MCP Server.
 * It checks for the OpenAI API key and starts the server.
 */

import { McpServer } from "@modelcontextprotocol/sdk/server/mcp.js";
import { StdioServerTransport } from "@modelcontextprotocol/sdk/server/stdio.js";
import { z } from "zod";
import { OpenAI } from "openai";
import { CodeAnalyzer } from "./code-analyzer.js";
import { TestFile } from "./code-analyzer.js";
import { TestStructureManager } from "./test-structure-manager.js";
import { MCPMessage, ListOfferingsResponse } from "./types.js";

// Initialize OpenAI client
let openai: OpenAI;

function getOpenAIClient(): OpenAI {
  if (!openai) {
    openai = new OpenAI({
      apiKey: process.env.OPENAI_API_KEY
    });
  }
  return openai;
}

// Create MCP server
const server = new McpServer({
  name: "Playwright Test Generator",
  version: "1.0.0"
});

// Add tool for listing test files
server.tool(
  "listTestFiles",
  { 
    directory: z.string(),
    pagePath: z.string()
  },
  async ({ directory, pagePath }) => {
    try {
      const testFiles = await CodeAnalyzer.findRelevantTestFiles(directory, pagePath);
      return {
        content: [{ 
          type: "text", 
          text: JSON.stringify(testFiles, null, 2)
        }]
      };
    } catch (error) {
      return {
        content: [{ 
          type: "text", 
          text: `Error listing test files: ${error instanceof Error ? error.message : 'Unknown error'}` 
        }],
        isError: true
      };
    }
  }
);

// Add tool for generating tests
server.tool(
  "generateTest",
  { 
    directory: z.string(),
    pagePath: z.string(),
    prompt: z.string()
  },
  async ({ directory, pagePath, prompt }) => {
    try {
      // Find relevant test files
      const { testFiles, codePatterns } = await CodeAnalyzer.analyzeTestFiles(directory, pagePath);

      // Generate test using OpenAI
      const response = await getOpenAIClient().chat.completions.create({
        model: "gpt-4",
        messages: [
          {
            role: "system",
            content: "You are a test generation assistant. Generate Playwright test code based on the provided patterns and requirements."
          },
          {
            role: "user",
            content: `
              Code Patterns:
              ${JSON.stringify(codePatterns, null, 2)}
              
              Requirements:
              ${prompt}
              
              Generate a Playwright test that follows these patterns and requirements.
            `
          }
        ]
      });

      const generatedTest = response.choices[0]?.message?.content || "";

      return {
        content: [{ 
          type: "text", 
          text: generatedTest
        }]
      };
    } catch (error) {
      return {
        content: [{ 
          type: "text", 
          text: `Error generating test: ${error instanceof Error ? error.message : 'Unknown error'}` 
        }],
        isError: true
      };
    }
  }
);

// Handle stdio communication
process.stdin.setEncoding('utf8');
process.stdin.on('data', (data: Buffer) => {
  try {
    const message = JSON.parse(data.toString());
    handleMCPMessage(message);
  } catch (error) {
    console.error('Error parsing MCP message:', error);
    sendErrorResponse(null, -32700, 'Parse error', 'Invalid JSON was received by the server');
  }
});

function sendResponse(id: string | number | null, result: any): void {
  const response = {
    jsonrpc: '2.0',
    id,
    result
  };
  process.stdout.write(JSON.stringify(response) + '\n');
}

function sendErrorResponse(id: string | number | null, code: number, message: string, data?: any): void {
  const response = {
    jsonrpc: '2.0',
    id,
    error: {
      code,
      message,
      data
    }
  };
  process.stdout.write(JSON.stringify(response) + '\n');
}

async function handleMCPMessage(message: any): Promise<void> {
  if (!message.jsonrpc || message.jsonrpc !== '2.0') {
    sendErrorResponse(message.id, -32600, 'Invalid Request', 'The JSON sent is not a valid Request object');
    return;
  }

  if (!message.method) {
    sendErrorResponse(message.id, -32600, 'Invalid Request', 'The method field is required');
    return;
  }

  if (message.method === 'ListOfferings') {
    const offerings = [
      {
        id: 'playwright-test',
        name: 'Playwright Test Generator',
        description: 'Generates Playwright tests based on natural language prompts',
        capabilities: ['test-generation']
      }
    ];
    sendResponse(message.id, { offerings });
  } else if (message.method === 'GenerateTest') {
    try {
      const { params } = message;
      if (!params || !params.directory || !params.pagePath) {
        sendErrorResponse(message.id, -32602, 'Invalid params', 'Missing required parameters: directory and pagePath');
        return;
      }

      const { directory, pagePath } = params;
      
      // Find relevant test files
      const testFiles = await CodeAnalyzer.findRelevantTestFiles(directory, pagePath);
      
      if (testFiles.length === 0) {
        sendResponse(message.id, { message: 'No relevant test files found' });
        return;
      }
      
      // Analyze test files for patterns
      const { testFiles: analyzedFiles, codePatterns } = await CodeAnalyzer.analyzeTestFiles(directory, pagePath);
      
      // Generate test using OpenAI
      const openai = getOpenAIClient();
      const response = await openai.chat.completions.create({
        model: "gpt-4",
        messages: [
          {
            role: "system",
            content: "You are a test generation assistant. Generate Playwright test code based on the provided patterns and requirements."
          },
          {
            role: "user",
            content: `
              Code Patterns:
              ${JSON.stringify(codePatterns, null, 2)}
              
              Requirements:
              ${params.prompt || 'Generate a test for the specified page'}
              
              Generate a Playwright test that follows these patterns and requirements.
            `
          }
        ]
      });

      const generatedTest = response.choices[0]?.message?.content || "";
      sendResponse(message.id, { test: generatedTest });
    } catch (error) {
      sendErrorResponse(message.id, -32000, 'Server error', error instanceof Error ? error.message : 'Unknown error');
    }
  } else {
    sendErrorResponse(message.id, -32601, 'Method not found', `The method '${message.method}' does not exist`);
  }
}

// Start server
function startServer(): void {
  console.log('MCP server ready to accept requests');
  sendResponse(null, { status: 'ready' });
}

startServer();

// Start the server
const transport = new StdioServerTransport();
await server.connect(transport); 