import express, { Request, Response, RequestHandler } from 'express';
import { chromium, Page } from 'playwright';
import fs from 'fs';
import path from 'path';
import { OpenAI } from 'openai';
import { glob } from 'glob';
import { AuthManager } from './auth-manager.js';
import { CodeAnalyzer } from './code-analyzer.js';
import { TestStructureManager } from './test-structure-manager.js';
import {
  MCPMessage,
  MCPResponse,
  ListOfferingsResponse,
  AuthConfig,
  SiteContext
} from './types.js';

interface PageInfo {
  path: string;
  url: string;
  elements: Record<string, string>;
}

interface PageState {
  html: string;
  screenshot: Buffer;
  title: string;
  url: string;
}

interface TestLocation {
  directory: string;
  fileName: string;
  fullPath: string;
}

interface CodePatterns {
  locators: Record<string, string>;
  fixtures: string[];
  helpers: Array<{
    name: string;
    code: string;
  }>;
  imports: string[];
  setupPatterns: string[];
  teardownPatterns: string[];
  assertionPatterns: string[];
  testStructure: {
    describe: string[];
    test: string[];
  };
  pageObjects: Array<{
    name: string;
    code: string;
  }>;
  commonImports: string[];
  fileStructure: Record<string, string[]>;
}

interface ProjectStructure {
  testDirectories: string[];
  fixtureFiles: string[];
  helperFiles: string[];
  pageObjectFiles: string[];
  configFiles: string[];
  patterns: {
    fileNaming: 'kebab-case' | 'dot.case' | 'camelCase' | 'PascalCase' | null;
    directoryOrganization: 'by-feature' | 'flat' | null;
    importPatterns: string[];
  };
}

interface TestFile {
  name: string;
  path: string;
  content: string;
  directory: string;
}

// Initialize OpenAI client
let openai: OpenAI;

function getOpenAIClient(): OpenAI {
  if (!openai) {
    openai = new OpenAI({
      apiKey: process.env.OPENAI_API_KEY
    });
  }
  return openai;
}

// Create Express app
const app = express();
app.use(express.json());

// Handle stdio communication
process.stdin.setEncoding('utf8');
process.stdin.on('data', (data: Buffer) => {
  try {
    const message: MCPMessage = JSON.parse(data.toString());
    handleMCPMessage(message);
  } catch (error) {
    console.error('Error parsing MCP message:', error);
  }
});

function handleMCPMessage(message: MCPMessage): void {
  if (message.type === 'ListOfferings') {
    const response: ListOfferingsResponse = {
      type: 'ListOfferingsResponse',
      timestamp: new Date().toISOString(),
      offerings: [
        {
          id: 'playwright-test',
          name: 'Playwright Test Generator',
          description: 'Generates Playwright tests based on natural language prompts',
          capabilities: ['test-generation']
        }
      ]
    };
    process.stdout.write(JSON.stringify(response) + '\n');
  }
}

// MCP endpoint
const mcpHandler = async (req: Request, res: Response): Promise<void> => {
  try {
    console.log('Received MCP request');
    const { prompt, workspaceRoot } = req.body;
    
    console.log(`Prompt: ${prompt}`);
    console.log(`Workspace root: ${workspaceRoot}`);
    
    // Initialize auth manager
    const authManager = new AuthManager(workspaceRoot);
    
    // Initialize test structure manager
    const testStructureManager = new TestStructureManager(workspaceRoot);
    
    // Load site context
    const siteContextPath = findSiteContextFile(workspaceRoot);
    if (!siteContextPath) {
      res.status(500).json({ 
        error: 'Site context file not found. Please create a site-context.json file in your project.' 
      });
      return;
    }
    
    console.log(`Loading site context from: ${siteContextPath}`);
    let siteContext: SiteContext;
    
    try {
      siteContext = JSON.parse(fs.readFileSync(siteContextPath, 'utf8'));
    } catch (error) {
      console.error(`Error loading site context: ${error instanceof Error ? error.message : 'Unknown error'}`);
      res.status(500).json({ error: 'Failed to load site context. Make sure it is valid JSON.' });
      return;
    }
    
    // Extract page from prompt
    console.log('Extracting page from prompt');
    const pageInfo = extractPageFromPrompt(prompt, siteContext);
    console.log(`Identified page: ${pageInfo.path}`);
    
    // Check if page requires authentication
    const authConfig = authManager.getAuthConfig();
    const requiresAuth = authManager.requiresAuthentication(pageInfo.url);
    console.log(`Page requires authentication: ${requiresAuth}`);
    
    // Capture current page state
    console.log(`Capturing current state of page: ${pageInfo.url}`);
    const pageState = await capturePageState(pageInfo.url, requiresAuth ? authConfig : null);
    
    // Analyze test files for code reuse
    console.log('Analyzing test files for code reuse');
    const { testFiles, codePatterns } = CodeAnalyzer.analyzeTestFiles(workspaceRoot, pageInfo.path);
    
    // Analyze project structure
    console.log('Analyzing project structure');
    const projectStructure = testStructureManager.analyzeProjectStructure();
    
    // Determine best location for the new test
    const testLocation = testStructureManager.generateTestPath(pageInfo.path, projectStructure);
    console.log(`Recommended test location: ${testLocation.fullPath}`);
    
    // Analyze locators to avoid duplication
    const locatorAnalysis = CodeAnalyzer.analyzeLocators(
      { ...pageInfo.elements, ...codePatterns.locators },
      pageState.html
    );
    
    // Generate MCP
    console.log('Generating MCP');
    const mcp = generateMCP(
      pageInfo, 
      pageState, 
      testFiles, 
      prompt, 
      codePatterns, 
      requiresAuth,
      testLocation,
      locatorAnalysis,
      projectStructure
    );
    
    // Return MCP
    console.log('Returning MCP to Cursor');
    res.json({ mcp });
  } catch (error) {
    console.error(`Error processing MCP request: ${error instanceof Error ? error.message : 'Unknown error'}`);
    res.status(500).json({ error: error instanceof Error ? error.message : 'Unknown error' });
  }
};

// Save generated test endpoint
const saveTestHandler = async (req: Request, res: Response): Promise<void> => {
  try {
    const { testCode, testLocation, workspaceRoot } = req.body;
    if (!testCode || !testLocation || !workspaceRoot) {
      res.status(400).json({ error: 'Missing required parameters' });
      return;
    }
    
    const testStructureManager = new TestStructureManager(workspaceRoot);
    const result = testStructureManager.saveTest(testCode, testLocation);
    
    if (!result.success) {
      res.status(500).json({ error: result.error });
      return;
    }
    
    res.json({
      message: result.message,
      path: result.path
    });
  } catch (error) {
    console.error(`Error saving test: ${error instanceof Error ? error.message : 'Unknown error'}`);
    res.status(500).json({ error: error instanceof Error ? error.message : 'Unknown error' });
  }
};

// Auth configuration endpoint
const authConfigHandler = (req: Request, res: Response): void => {
  try {
    const { workspaceRoot, config } = req.body;
    if (!workspaceRoot) {
      res.status(400).json({ error: 'Workspace root is required' });
      return;
    }
    
    const authManager = new AuthManager(workspaceRoot);
    
    if (config) {
      // Update auth config
      const success = authManager.saveAuthConfig(config);
      if (!success) {
        res.status(500).json({ error: 'Failed to save auth configuration' });
        return;
      }
      res.json({ message: 'Authentication configuration updated successfully' });
    } else {
      // Get current auth config
      const authConfig = authManager.getAuthConfig();
      res.json({ config: authConfig });
    }
  } catch (error) {
    console.error(`Error handling auth config: ${error instanceof Error ? error.message : 'Unknown error'}`);
    res.status(500).json({ error: error instanceof Error ? error.message : 'Unknown error' });
  }
};

// Create sample auth config endpoint
const createSampleAuthHandler = (req: Request, res: Response): void => {
  try {
    const { workspaceRoot } = req.body;
    if (!workspaceRoot) {
      res.status(400).json({ error: 'Workspace root is required' });
      return;
    }
    
    const authManager = new AuthManager(workspaceRoot);
    const success = authManager.createSampleAuthConfig();
    
    if (!success) {
      res.status(500).json({ error: 'Failed to create sample auth configuration' });
      return;
    }
    
    res.json({
      message: 'Sample authentication configuration created successfully',
      config: authManager.getAuthConfig()
    });
  } catch (error) {
    console.error(`Error creating sample auth config: ${error instanceof Error ? error.message : 'Unknown error'}`);
    res.status(500).json({ error: error instanceof Error ? error.message : 'Unknown error' });
  }
};

// Register routes
app.post('/mcp', mcpHandler);
app.post('/save-test', saveTestHandler);
app.post('/auth-config', authConfigHandler);
app.post('/create-sample-auth', createSampleAuthHandler);

// Helper function to find site context file
function findSiteContextFile(workspaceRoot: string): string | null {
  // Look for site-context.json in various locations
  const possiblePaths = [
    path.join(workspaceRoot, 'site-context.json'),
    path.join(workspaceRoot, 'site-context.txt'),
    path.join(workspaceRoot, 'data', 'site-context.json'),
    path.join(workspaceRoot, 'config', 'site-context.json'),
    path.join(workspaceRoot, 'playwright-mcp-system', 'site-context.json')
  ];
  
  for (const filePath of possiblePaths) {
    if (fs.existsSync(filePath)) {
      return filePath;
    }
  }
  
  return null;
}

// Helper function to extract page from prompt
function extractPageFromPrompt(prompt: string, siteContext: SiteContext): PageInfo {
  // Extract page path from prompt
  const pagePath = prompt.split(' ')[0];
  
  // Get page info from site context
  const pageInfo = siteContext.pages[pagePath];
  if (!pageInfo) {
    throw new Error(`Page not found in site context: ${pagePath}`);
  }
  
  return {
    path: pagePath,
    url: `${siteContext.baseURL}${pagePath}`,
    elements: pageInfo.elements
  };
}

// Helper function to capture page state
async function capturePageState(url: string, authConfig: AuthConfig | null = null): Promise<PageState> {
  const browser = await chromium.launch();
  const context = await browser.newContext();
  const page = await context.newPage();
  
  try {
    // Navigate to the page
    await page.goto(url);
    
    // Handle authentication if required
    if (authConfig) {
      await performLogin(page, authConfig);
    }
    
    // Wait for the page to be fully loaded
    await page.waitForLoadState('networkidle');
    
    // Capture page state
    const html = await page.content();
    const screenshot = await page.screenshot();
    const title = await page.title();
    
    return {
      html,
      screenshot,
      title,
      url: page.url()
    };
  } finally {
    await browser.close();
  }
}

// Helper function to perform login
async function performLogin(page: Page, authConfig: AuthConfig): Promise<void> {
  // Navigate to login page
  await page.goto(authConfig.loginUrl);
  
  // Fill in login form
  await page.fill(authConfig.usernameSelector, authConfig.username);
  await page.fill(authConfig.passwordSelector, authConfig.password);
  
  // Submit form
  await page.click(authConfig.submitSelector);
  
  // Wait for navigation
  await page.waitForNavigation();
}

// Helper function to generate MCP
function generateMCP(
  pageInfo: PageInfo,
  pageState: PageState,
  testFiles: TestFile[],
  prompt: string,
  codePatterns: CodePatterns,
  requiresAuth: boolean,
  testLocation: TestLocation,
  locatorAnalysis: Record<string, string>,
  projectStructure: ProjectStructure
): Record<string, any> {
  return {
    type: 'GenerateTestResponse',
    timestamp: new Date().toISOString(),
    testCode: '',
    location: {
      directory: testLocation.directory,
      fileName: testLocation.fileName
    },
    metadata: {
      pageInfo,
      pageState,
      testFiles,
      prompt,
      codePatterns,
      requiresAuth,
      locatorAnalysis,
      projectStructure
    }
  };
}

// Start the server
function startServer(): void {
  const port = process.env.PORT || 3000;
  app.listen(port, () => {
    console.log(`MCP server listening on port ${port}`);
  });
}

export { startServer }; 