/**
 * Test Structure Manager for Playwright MCP Server
 * 
 * This module handles the integration of generated tests into the existing project structure,
 * ensuring they follow the same patterns and conventions as existing tests.
 */

import fs from 'fs';
import path from 'path';

interface TestLocation {
  directory: string;
  fileName: string;
  fullPath: string;
}

interface SaveTestResult {
  success: boolean;
  path?: string;
  message?: string;
  error?: string;
}

interface ProjectStructureAnalysis {
  testDirectories: string[];
  fixtureFiles: string[];
  helperFiles: string[];
  pageObjectFiles: string[];
  configFiles: string[];
  patterns: {
    fileNaming: NamingConvention | null;
    directoryOrganization: DirectoryOrganization | null;
    importPatterns: string[];
  };
}

type NamingConvention = 'kebab-case' | 'dot.case' | 'camelCase' | 'PascalCase';
type DirectoryOrganization = 'by-feature' | 'flat';

/**
 * Test Structure Manager class
 */
export class TestStructureManager {
  private workspaceRoot: string;

  constructor(workspaceRoot: string) {
    this.workspaceRoot = workspaceRoot;
  }

  /**
   * Saves a generated test to the appropriate location
   * @param testCode - The generated test code
   * @param testLocation - Object containing directory and fileName
   * @returns Result of the save operation
   */
  public saveTest(testCode: string, testLocation: TestLocation): SaveTestResult {
    try {
      // Ensure the directory exists
      const fullDir = path.join(this.workspaceRoot, testLocation.directory);
      if (!fs.existsSync(fullDir)) {
        fs.mkdirSync(fullDir, { recursive: true });
      }

      // Save the test file
      const fullPath = path.join(this.workspaceRoot, testLocation.fullPath);
      fs.writeFileSync(fullPath, testCode);

      return {
        success: true,
        path: fullPath,
        message: `Test saved to ${fullPath}`
      };
    } catch (error) {
      console.error(`Error saving test: ${error instanceof Error ? error.message : 'Unknown error'}`);
      return {
        success: false,
        error: error instanceof Error ? error.message : 'Unknown error'
      };
    }
  }

  /**
   * Analyzes the project structure to determine test organization patterns
   * @returns Analysis of the project's test structure
   */
  public analyzeProjectStructure(): ProjectStructureAnalysis {
    const analysis: ProjectStructureAnalysis = {
      testDirectories: [],
      fixtureFiles: [],
      helperFiles: [],
      pageObjectFiles: [],
      configFiles: [],
      patterns: {
        fileNaming: null,
        directoryOrganization: null,
        importPatterns: []
      }
    };

    // Find test directories
    const possibleTestDirs = [
      path.join(this.workspaceRoot, 'tests'),
      path.join(this.workspaceRoot, 'test'),
      path.join(this.workspaceRoot, 'src', 'tests'),
      path.join(this.workspaceRoot, 'e2e')
    ];

    for (const dir of possibleTestDirs) {
      if (fs.existsSync(dir)) {
        analysis.testDirectories.push(dir);
        this.analyzeTestDirectory(dir, analysis);
      }
    }

    // Determine file naming pattern
    if (analysis.patterns.fileNaming === null) {
      // Default to kebab-case if we couldn't determine
      analysis.patterns.fileNaming = 'kebab-case';
    }

    return analysis;
  }

  /**
   * Analyzes a test directory to extract patterns
   * @param directory - Directory to analyze
   * @param analysis - Analysis object to update
   */
  private analyzeTestDirectory(directory: string, analysis: ProjectStructureAnalysis): void {
    try {
      const files = fs.readdirSync(directory);
      
      // Check for subdirectories
      const subdirs = files.filter(file => {
        const fullPath = path.join(directory, file);
        return fs.statSync(fullPath).isDirectory();
      });
      
      if (subdirs.length > 0) {
        // Project likely organizes tests by feature/module
        analysis.patterns.directoryOrganization = 'by-feature';
        
        // Recursively analyze subdirectories
        for (const subdir of subdirs) {
          this.analyzeTestDirectory(path.join(directory, subdir), analysis);
        }
      }
      
      // Analyze test files
      const testFiles = files.filter(file => {
        return file.endsWith('.spec.ts') || file.endsWith('.test.ts');
      });
      
      if (testFiles.length > 0) {
        // Determine file naming convention
        if (testFiles[0].includes('-')) {
          analysis.patterns.fileNaming = 'kebab-case';
        } else if (testFiles[0].includes('.')) {
          analysis.patterns.fileNaming = 'dot.case';
        } else if (testFiles[0][0] === testFiles[0][0].toLowerCase()) {
          analysis.patterns.fileNaming = 'camelCase';
        } else {
          analysis.patterns.fileNaming = 'PascalCase';
        }
        
        // Look for fixture, helper, and page object files
        for (const file of files) {
          const fullPath = path.join(directory, file);
          if (fs.statSync(fullPath).isFile()) {
            if (file.includes('fixture') || file.includes('setup')) {
              analysis.fixtureFiles.push(fullPath);
            } else if (file.includes('helper') || file.includes('util')) {
              analysis.helperFiles.push(fullPath);
            } else if (file.includes('page') && !file.endsWith('.spec.ts') && !file.endsWith('.test.ts')) {
              analysis.pageObjectFiles.push(fullPath);
            } else if (file === 'playwright.config.ts' || file === 'playwright.config.js') {
              analysis.configFiles.push(fullPath);
            }
          }
        }
        
        // Analyze import patterns in the first test file
        const firstTestFile = path.join(directory, testFiles[0]);
        const content = fs.readFileSync(firstTestFile, 'utf8');
        
        const importRegex = /import\s+.*?from\s+['"]([^'"]+)['"]/g;
        let match;
        while ((match = importRegex.exec(content)) !== null) {
          analysis.patterns.importPatterns.push(match[0]);
        }
      }
    } catch (error) {
      console.error(`Error analyzing test directory ${directory}: ${error instanceof Error ? error.message : 'Unknown error'}`);
    }
  }

  /**
   * Formats a test name according to the project's naming convention
   * @param testName - The raw test name
   * @param convention - The naming convention to use
   * @returns Formatted test name
   */
  private formatTestName(testName: string, convention: NamingConvention = 'kebab-case'): string {
    // Remove special characters and spaces
    let formatted = testName.replace(/[^\w\s]/g, '');
    
    switch (convention) {
      case 'kebab-case':
        return formatted.toLowerCase().replace(/\s+/g, '-');
      case 'dot.case':
        return formatted.toLowerCase().replace(/\s+/g, '.');
      case 'camelCase':
        return formatted.replace(/\s+(.)/g, (_, char) => char.toUpperCase())
          .replace(/\s+/g, '')
          .replace(/^(.)/, (_, char) => char.toLowerCase());
      case 'PascalCase':
        return formatted.replace(/\s+(.)/g, (_, char) => char.toUpperCase())
          .replace(/\s+/g, '')
          .replace(/^(.)/, (_, char) => char.toUpperCase());
      default:
        return formatted.toLowerCase().replace(/\s+/g, '-');
    }
  }

  /**
   * Generates a test file path based on project structure analysis
   * @param pagePath - The path of the page being tested
   * @param analysis - Project structure analysis
   * @returns Recommended test location
   */
  public generateTestPath(pagePath: string, analysis: ProjectStructureAnalysis): TestLocation {
    // Extract page name from path
    const pageName = pagePath.split('/').filter(Boolean).pop() || 'home';
    
    // Format the file name according to the project's convention
    const fileName = this.formatTestName(pageName, analysis.patterns.fileNaming || 'kebab-case') + 
      (analysis.patterns.fileNaming === 'PascalCase' ? 'Spec.ts' : '.spec.ts');
    
    // Determine the directory
    let directory;
    if (analysis.patterns.directoryOrganization === 'by-feature' && analysis.testDirectories.length > 0) {
      // Try to find or create a feature directory based on the page name
      directory = path.join(
        path.relative(this.workspaceRoot, analysis.testDirectories[0]),
        this.formatTestName(pageName, 'kebab-case')
      );
    } else if (analysis.testDirectories.length > 0) {
      // Use the first test directory
      directory = path.relative(this.workspaceRoot, analysis.testDirectories[0]);
    } else {
      // Default to 'tests' directory
      directory = 'tests';
    }
    
    return {
      directory,
      fileName,
      fullPath: path.join(directory, fileName)
    };
  }
} 